<?php

namespace SenSai;

use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Commands\CommandListener;

/**
 * Loadscreens Plugin
 *
 * @author Sen-Sai
 */

class Loadscreens implements CallbackListener, TimerListener, CommandListener, Plugin {
    /**
     * Constants
     */
    const PLUGIN_ID     = 45;
    const PLUGIN_VERSION = 1.14;
    const PLUGIN_NAME = 'Loadscreens';
    const PLUGIN_AUTHOR = 'G]R [SG-1]Sen-sai';

    // Loadscreen properties

    const MLID_LOADSCREENS     = 'Loadscreens.ID';
    const SETTING_LS_ACTIVATED = 'Loadscreens activated';
    const SETTING_LS_POSX      = 'Loadscreens Position: X';
    const SETTING_LS_POSY      = 'Loadscreens Position: Y';
    const SETTING_LS_WIDTH     = 'Loadscreens Size: Width';
    const SETTING_LS_HEIGHT    = 'Loadscreens Size: Height';
    const SETTING_LS_URL       = 'Loadscreens URL';
    const SETTING_LS_OFFSET    = 'Loadscreens EndMatch Delay';

    private $visible = false;
	private $screen = array();
	private $cnt = 0;

    /**
     * Private Properties
     */
    /**
     * @var maniaControl $maniaControl
     */
    private $maniaControl = null;
	private $mapbegon = true;

    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     * @return mixed
     */
    public static function prepare(ManiaControl $maniaControl) {
        //do nothing
    }

    /**
     * Load the plugin
     *
     * @param ManiaControl $maniaControl
     * @return bool
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;

        //Register callbacks

        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_ACTIVATED, true);
        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_URL, 'http://files.designburo.nl/sm/loadscreens.xml');
        $this->maniaControl->settingManager->initSetting($this, self::SETTING_LS_OFFSET, 15);
		
		$file = $this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_URL);
		$array = get_headers($file);
		$string = $array[0];
		if(!strpos($string,"200")) {
			
			$this->maniaControl->chat->sendInformation('The URL does not exist. Please check. Loadscreens disabled.', null, false);
			return;
		}
        $this->maniaControl->callbackManager->registerCallbackListener(Callbacks::BEGINMAP, $this, 'handleOnBeginMap');
		$this->maniaControl->callbackManager->registerCallbackListener(Callbacks::BEGINROUND, $this, 'handleOnBeginMap');
		$this->maniaControl->callbackManager->registerCallbackListener(Callbacks::ENDMAP, $this, 'handleOnEndMap');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'preloadtoLogin');
		$this->maniaControl->commandManager->registerCommandListener('hidescreen', $this, 'hideScreen', true);
		
		$ext = substr(strrchr($file, "."), 1);
		if ($ext=="xml")
		{
			if ($xml = simplexml_load_string(file_get_contents($file)))
			{
				foreach($xml->loadscreen_image as $screen)
				{
					$this->screen[] = $screen;
				}
			}
			$this->cnt = count($this->screen);
			$this->maniaControl->chat->sendInformation('Got '.$this->cnt.' screens', null, false);
		}
		elseif ($ext=="jpg" || $ext=="png")
		{
			$this->cnt=1;
			$this->screen = $file;
		}
		else
		{
			$this->maniaControl->chat->sendInformation('The URL must be either .xml, .png or .jpg', null, false);
			$this->maniaControl->callbackManager->unregisterCallbackListener($this);
			return;	
		}
		$this->preload();
    }
	public function preloadtoLogin(player $player) {
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = -2000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->add($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);

		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->add($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->add($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);	
			$quad->setVisible(false);		
		}
		$this->maniaControl->manialinkManager->sendManialink($manialink,$player);
	}
	
	public function preload() {
		//$login=$player->login;
        $pos_x = 128;
        $pos_y = 128;
		$pos_z = -2000;
        $width = 128;
        $height = 180;
		$manialink = new ManiaLink(self::MLID_LOADSCREENS);
		$frame = new Frame();
        $manialink->add($frame);
        $frame->setPosition($pos_x, $pos_y);
		$frame->setSize($width, $height);
		if ($this->cnt > 1)
		{
			foreach ($this->screen as $image)
			{
				$quad = new Quad();
        		$frame->add($quad);
        		$quad->setSize($width, $height);
        		$quad->setImage($image);
				$quad->setVisible(false);
			}
		}
		else
		{
			$image= $this->screen;
			$quad = new Quad();
			$frame->add($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);	
			$quad->setVisible(false);		
		}
		$this->maniaControl->chat->sendInformation('Preloading screens to all.', null, false);
		$this->maniaControl->manialinkManager->sendManialink($manialink);
	}
	
    public function displayLoadscreens(){
		if ($this->mapbegon) $this->hideScreen();
        $pos_x = 0;
        $pos_y = 0;
        $width = 320;
        $height = 180;
		if($this->cnt > 1)
		{
			$nr=mt_rand(1,($this->cnt-1));
			$image  = $this->screen[$nr];

		}
		else
		{
        	$image  = $this->screen;
		}
		$ext = substr(strrchr($image, "."), 1);
        $manialink = new ManiaLink(self::MLID_LOADSCREENS);

        $frame = new Frame();
        $manialink->add($frame);
        $frame->setSize($width, $height);
        $frame->setPosition($pos_x, $pos_y);

			$quad = new Quad();
			$frame->add($quad);
			$quad->setSize($width, $height);
			$quad->setImage($image);
			$quad->setZ(25.0);
        
        $this->maniaControl->manialinkManager->sendManialink($manialink);
    }

    public function hideScreen() {
        $emptyManialink = new ManiaLink(self::MLID_LOADSCREENS);
        $this->maniaControl->manialinkManager->sendManialink($emptyManialink);
    }

    public function hideLoadscreens($widgetId) {
        $emptyManialink = new ManiaLink($widgetId);
        $this->maniaControl->manialinkManager->sendManialink($emptyManialink);
    }

    public function handleOnBeginMap() {
        if($this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_ACTIVATED))
		{
            $this->hideLoadscreens(self::MLID_LOADSCREENS);
			$this->mapbegon = true;
        }
    }

    public function handleOnEndMap() {
        if($this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_ACTIVATED))
		{
            $this->maniaControl->timerManager->registerTimerListening($this, 'displayLoadscreens', $this->maniaControl->settingManager->getSetting($this, self::SETTING_LS_OFFSET)*1000,true);
			$this->mapbegon = false;
        }
    }

    public function unload() {
        $this->hideLoadscreens(self::MLID_LOADSCREENS);
        $this->maniaControl->callbackManager->unregisterCallbackListener($this);
        unset($this->maniaControl);
    }

     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::PLUGIN_ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::PLUGIN_NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::PLUGIN_VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::PLUGIN_AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return 'Displays a custom images during map changes.';
    }
}